package de.akquinet.jbpm5demo.myWorkflow.testing.examples;

import java.util.ArrayList;
import java.util.List;

import org.drools.runtime.StatefulKnowledgeSession;
import org.jbpm.task.TaskService;
import org.jbpm.test.JbpmJUnitTestCase;
import org.jbpm.workflow.instance.WorkflowProcessInstance;
import org.junit.Test;

/** 
 * Ein Beispielprozess mit zwei HumanTask, die jeweils von zwei unterschiedlichen Gruppen ausgefhrt werden.
 * Dabei werden jeweils 2 Parameter weitergegeben und es werden mit jedem Human Task die Daten verndert.
 * group1 = Human Task 1 (humanTask1)
 * group2 = Human Task 2 (humanTask2)
 * 
 * Bevor ein task gestartet werden kann, muss diser anhand der GruppeId (GroupId) gesucht werden.
 * Beim starten des Tasks ist darauf zu achten, dass dieser erst mit claim an den auszufhrenden User gebunden wird.
 * 
 * @author Pascal Schaerf
 */
public class MySimpleTwoHumanTaskTwoGroupsExample extends JbpmJUnitTestCase {

	private static final String PROCESS_FILE_NAME = "mySimpleTwoHumanTaskTwoGroupsExample.bpmn";
	private static final String PROCESS_ID = "mySimpleTwoHumanTaskTwoGroupsExample";
	
	public MySimpleTwoHumanTaskTwoGroupsExample() {
		super(true);
	}
	
	private static final String USER_1 = "user1";
	private static final String USER_2 = "user2";
	
	private static final String GROUP_1 = "blubb";
	private static final String GROUP_2 = "group2";
	
	private static final List<String> GROUP_LIST_1 = new ArrayList<String>();
	private static final List<String> GROUP_LIST_2 = new ArrayList<String>();	
	
	@Test
	public void testProcess() {
		GROUP_LIST_1.add(GROUP_1);
		GROUP_LIST_2.add(GROUP_2);
		StatefulKnowledgeSession ksession = createKnowledgeSession(PROCESS_FILE_NAME);
		TaskService taskService = getTaskService(ksession);
		WorkflowProcessInstance processInstance  = (WorkflowProcessInstance) ksession.startProcess(PROCESS_ID);
		assertProcessInstanceActive(processInstance.getId(), ksession);

		assertNodeTriggered(processInstance.getId(), "StartProcess");
		
		assertNodeTriggered(processInstance.getId(), "Human Task 1");
		long taskId =taskService.getTasksAssignedAsPotentialOwner(USER_1, GROUP_LIST_1, "en-UK").get(0).getId();
		taskService.claim(taskId, USER_1, GROUP_LIST_1);
		taskService.start(taskId, USER_1);		
		taskService.complete(taskId, USER_1, null);
	
		
		assertNodeTriggered(processInstance.getId(), "Human Task 2");
		long task2Id = taskService.getTasksAssignedAsPotentialOwner(USER_2, GROUP_LIST_2, "en-UK").get(0).getId();
		taskService.claim(task2Id, USER_2, GROUP_LIST_2);
		taskService.start(task2Id, USER_2);		
		taskService.complete(task2Id, USER_2, null);		

		assertProcessInstanceCompleted(processInstance.getId(), ksession);
		ksession.dispose();
	}
}